import React from "react";
import { Button, Col, Container, Form, Row } from "react-bootstrap";
import { Controller, useForm } from "react-hook-form";
import { useTranslation } from "react-i18next";
import { useNavigate } from "react-router";
import validator from "validator";
import Divider from "../components/Divider";
import Modal from "../components/Modal";

type FormData = {
  sport: string;
  boatName: string;
  startTime: string;
  estimatedEndTime: string;
  destination: string;
  name: string;
  email: string;
  annotations: string;
};

function Book() {
  let date = new Date();
  const zeroPad = (num: number, places: number) => String(num).padStart(places, '0')
  const navigate = useNavigate();
  const {
    control,
    handleSubmit,
    formState: { errors },
  } = useForm<FormData>({
    mode: "onBlur", defaultValues: {
      sport: "Sport2",
      boatName: "Boot1",
      startTime: zeroPad(date.getHours(), 2) + ":" + zeroPad(date.getMinutes(), 2),
      estimatedEndTime: zeroPad(date.getHours() + 2, 2) + ":" + zeroPad(date.getMinutes(), 2),
      destination: "",
      name: "",
      // annotations: ""
    }
  });
  const { t } = useTranslation(); // i18n
  const onSubmit = (data: FormData) => {
    try {
      /**
       * Logic to handle booking creation. Should be separated into a service in another file.
       * await createBooking(data);
       */
      navigate("/book/success");
    } catch (e) {
      alert("error");
    }
  };
  return (
    <Container className="pt-4">
      <Row>
        <Col xs={{ span: 10, offset: 1 }}>
          <Modal>
            <h1 className="text-center">{t("bookingForm.title")}</h1>
            <p className="text-center">{t("bookingForm.subtitle")}</p>
            <Divider />
            <Form onSubmit={handleSubmit(onSubmit)}>
              <Controller
                name="sport"
                control={control}
                render={({ field }) => (
                  <div className="mb-2 required">
                    <Form.Label>{t("bookingForm.labelSport")}</Form.Label>
                    <Form.Select aria-label="Default select example" {...field}>
                      {["Sport1", "Sport2", "Sport3"].map((b) => (
                        <option key={b}>{b}</option>
                      ))}
                    </Form.Select>
                  </div>
                )}
                rules={{
                  required: {
                    value: true,
                    message: t("bookingForm.messages.required", {
                      val: t("bookingForm.labelSport"),
                    }),
                  },
                }}
              />
              <Controller
                name="boatName"
                control={control}
                render={({ field }) => (
                  <div className="mb-2 required">
                    <Form.Label>{t("bookingForm.labelBoatName")}</Form.Label>
                    <Form.Select aria-label="Default select example" {...field}>
                      {["Boot1", "Boot2", "Boot3"].map((b) => (
                        <option key={b}>{b}</option>
                      ))}
                    </Form.Select>
                  </div>
                )}
                rules={{
                  required: {
                    value: true,
                    message: t("bookingForm.messages.required", {
                      val: t("bookingForm.labelBoatName"),
                    }),
                  },
                }}
              />
              <Controller
                name="startTime"
                control={control}
                render={({ field }) => (
                  <div className="mb-2 required">
                    <Form.Label>{t("bookingForm.labelStartTime")}</Form.Label>
                    <Form.Control type="time" {...field} />
                  </div>
                )}
                rules={{
                  required: {
                    value: true,
                    message: t("bookingForm.messages.required", {
                      val: t("bookingForm.labelStartTime"),
                    }),
                  },
                  pattern: {
                    value: /(((0|1)\d)|(2[0-3])):[0-5]\d/,
                    message: t("bookingForm.messages.invalidTime", {
                      val: t("bookingForm.labelStartTime"),
                    }),
                  },
                }}
              />
              <Controller
                name="estimatedEndTime"
                control={control}
                render={({ field }) => (
                  <div className="mb-2 required">
                    <Form.Label>
                      {t("bookingForm.labelEstimatedEndTime")}
                    </Form.Label>
                    <Form.Control type="time" {...field} />
                  </div>
                )}
                rules={{
                  required: {
                    value: true,
                    message: t("bookingForm.messages.required", {
                      val: t("bookingForm.labelEstimatedEndTime"),
                    }),
                  },
                  pattern: {
                    value: /(((0|1)\d)|(2[0-3])):[0-5]\d/,
                    message: t("bookingForm.messages.invalidTime", {
                      val: t("bookingForm.labelEstimatedEndTime"),
                    }),
                  },
                }}
              />
              <Controller
                name="destination"
                control={control}
                defaultValue=""
                render={({ field }) => (
                  <div className="mb-2 required">
                    <Form.Label>{t("bookingForm.labelDestination")}</Form.Label>
                    <Form.Control type="text" {...field} isInvalid={!!errors.destination}/>
                    <Form.Control.Feedback type="invalid">
                      {errors.destination?.message}
                    </Form.Control.Feedback>
                  </div>
                )}
                rules={{
                  required: {
                    value: true,
                    message: t("bookingForm.messages.required", {
                      val: t("bookingForm.labelDestination"),
                    }),
                  },
                }}
              />
              <Controller
                name="email"
                control={control}
                defaultValue=""
                render={({ field }) => (
                  <div className="mb-2 required">
                    <Form.Label>{t("bookingForm.labelEmail")}</Form.Label>
                    <Form.Control type="email" {...field} isInvalid={!!errors.email}/>
                    <Form.Control.Feedback type="invalid">
                      {errors.email?.message}
                    </Form.Control.Feedback>
                  </div>
                )}
                rules={{
                  required: {
                    value: true,
                    message: t("bookingForm.messages.required", {
                      val: t("bookingForm.labelEmail"),
                    }),
                  },
                  validate: (value: string) =>
                    validator.isEmail(value) ||
                    t("bookingForm.messages.invalidEmail").toString(),
                }}
              />
              <Controller
                name="name"
                control={control}
                defaultValue=""
                render={({ field }) => (
                  <div className="mb-2 required">
                    <Form.Label>{t("bookingForm.labelName")}</Form.Label>
                    <Form.Control type="text" {...field} isInvalid={!!errors.name}/>
                    <Form.Control.Feedback type="invalid">
                      {errors.name?.message}
                    </Form.Control.Feedback>
                  </div>
                )}
                rules={{
                  required: {
                    value: true,
                    message: t("bookingForm.messages.required", {
                      val: t("bookingForm.labelName"),
                    }),
                  },
                }}
              />
              {/* <Controller
                name="annotations"
                control={control}
                defaultValue=""
                render={({ field }) => (
                  <div className="mb-2">
                    <Form.Label>{t("bookingForm.labelAnnotations")}</Form.Label>
                    <Form.Control as="textarea" type="text" {...field} />
                  </div>
                )}
              /> */}
              <Button type="submit" variant="secondary" className="mt-2 w-100">
                {t("bookingForm.buttonBookBoat")}
              </Button>
              {/* <div className="mt-2">
                {Object.values(errors).map((error) => (
                  <p key={error.message} className="m-0 text-center text-danger">{error.message}</p>
                ))}
              </div> */}
            </Form>
          </Modal>
        </Col>
      </Row>
    </Container>
  );
}

export default Book;
