import bcrypt from "bcrypt";
import { Request, Response } from "express";
import Employee from "../db/models/Employee";
//create new account
const createAccountController = async (req: Request, res: Response) => {
  try {
    if (!(res.locals.user.role === "coordinator")) {
      return res
        .status(403)
        .json({ success: false, error: "MustBeCoordinator" });
    }

    const { first_name, last_name, email, password, role } = req.body;

    const existedAccount = await Employee.findAll({
      where: {
        email: email,
      },
    });

    if (existedAccount.length > 0) {
      return res
        .status(409)
        .json({ success: false, error: "AccountAlreadyExists" });
    }

    const hashedPassword = await bcrypt.hash(password, 10);

    const newAccount = await Employee.create({
      email,
      first_name: first_name,
      last_name: last_name,
      password: hashedPassword,
      role,
    });

    return res.status(201).send({
      success: true,
      account: {
        first_name: newAccount.first_name,
        last_name: newAccount.last_name,
        email: newAccount.email,
        role: newAccount.role,
      },
    });
  } catch {
    return res.status(500).json({ success: false, error: "serverError" });
  }
};
//show all accounts
const showAllAccounts = async (req: Request, res: Response) => {
  try {
    if (!(res.locals.user.role === "coordinator")) {
      return res
        .status(403)
        .json({ success: false, error: "MustBeCoordinator" });
    }
    const allAccounts = await Employee.findAll();
    return res.status(200).send({
      success: true,
      result: allAccounts.map((account) => {
        return {
          id: account.id,
          first_name: account.first_name,
          last_name: account.last_name,
          email: account.email,
          role: account.role,
        };
      }),
    });
  } catch (error) {
    console.error("server error: ", error.message);
    return res.status(500).json({ success: false, error: "serverError" });
  }
};

//show a specific account using given id: email is the primary key for a Employee
const showAccountById = async (req: Request, res: Response) => {
  try {
    if (!(res.locals.user.role === "coordinator")) {
      return res
        .status(403)
        .json({ success: false, error: "MustBeCoordinator" });
    }
    const givenId = req.params.id;
    const account = await Employee.findByPk(givenId);
    if (account) {
      return res.status(200).json({
        success: true,
        result: {
          id: account.id,
          first_name: account.first_name,
          last_name: account.last_name,
          email: account.email,
          role: account.role,
        },
      });
    }
    return res.status(404).json({ success: false, error: "accountIdNotFound" });
  } catch (error) {
    console.error("server error: ", error.message);
    return res.status(500).json({ success: false, error: "serverError" });
  }
};

//update account by id
const updateAccount = async (req: Request, res: Response) => {
  //input needs validation: handleValidationResult!?
  //by trying to update email, duplicates could be found which leads to server-error: must be handled seperately?

  try {
    if (!(res.locals.user.role === "coordinator")) {
      return res
        .status(403)
        .json({ success: false, error: "MustBeCoordinator" });
    }

    //take what ever in req.body is, and pass it to update()
    const input = req.body;
    const givenId = req.params.id;

    //check if given ID exists in DB
    const checkIfIdExists = await Employee.findOne({
      attributes: ["id"],
      where: {
        id: givenId,
      },
    });

    if (checkIfIdExists === null) {
      return res.status(404).json({ success: false, error: "accountNotFound" });
    }

    //hash given password
    if (input.password !== undefined) {
      input.password = await bcrypt.hash(input.password, 10);
    }

    //update Employee
    const userDataAfterUpdate = await Employee.update(input, {
      where: {
        id: givenId, //primary key in Employee is email
      },
      returning: true,
    });

    const userData = userDataAfterUpdate[1][0];
    delete userData.password;

    return res.status(200).json({
      success: true,
      result: {
        id: userData.id,
        first_name: userData.first_name,
        last_name: userData.last_name,
        email: userData.email,
        role: userData.role,
      },
    });
  } catch (error) {
    console.error("server error: ", error.message);
    return res.status(500).json({ success: false, error: "serverError" });
  }
};

//delete specific account using given id
const deleteAccountById = async (req: Request, res: Response) => {
  //what happens by deleting own-account (coordinator-account)?

  try {
    //check authority
    if (!(res.locals.user.role === "coordinator")) {
      return res
        .status(403)
        .json({ success: false, error: "MustBeCoordinator" });
    }
    //delete using given id (email)
    const givenId = req.params.id;
    const accountToDelete = await Employee.destroy({
      where: {
        id: givenId,
      },
    });

    if (accountToDelete == 0) {
      return res
        .status(404)
        .json({ success: false, error: "accountIdDoesNotExist" });
    }

    return res.status(204).json({ success: true });
  } catch (error) {
    console.error("server error: ", error.message);
    return res.status(500).json({ success: false, error: "serverError" });
  }
};

const accountsControllers = {
  createAccountController,
  showAllAccounts,
  showAccountById,
  updateAccount,
  deleteAccountById,
};

export default accountsControllers;
